#!/usr/bin/perl
#
# robo.pl - robot-arm controlling program for paico
#

use Tk;

$version = "0.1";
$author = "Benjamin Fuhrmannek <fuhrmannek\@web.de>";


### show the Info-Box
sub info {
  $infobutton->configure(-relief => "sunken");
  while (!$info->Show eq "OK") {}
  $infobutton->configure(-relief => "raised");
}

### change the button-relief
sub ch_relief {
  my ($bnum, $relief) = @_;
  if ($relief == 0) { $relief = "raised"; }
  if ($relief == 1) { $relief = "sunken"; }
  $button[$bnum]->configure(-relief => $relief);
}

sub get_relief {
  my ($bnum) = @_;
  my $relief = 0;
  if ($button[$bnum]->cget(-relief) eq "sunken") { $relief = 1; }
  $relief;
}

sub update_relief {
  my $i;
  for ($i = 2; $i <= 6; $i++) {
    if ($bit[1] == 0 && $bit[$i] != &get_relief($i)) { &ch_relief($i,$bit[$i]); }
    if ($bit[1] == 1 && &get_relief($i) != 0) { &ch_relief($i,0); }
  }
  for ($i = 12; $i <= 16; $i++) {
    if ($bit[1] == 1 && $bit[$i-10] != &get_relief($i)) { &ch_relief($i,$bit[$i-10]); }
    if ($bit[1] == 0 && &get_relief($i) != 0) { &ch_relief($i,0); }
  }
  if ($bit[7] != &get_relief(7)) { &ch_relief(7,$bit[7]) }
}

###
sub update_statusbar {
  $statusbar->configure(-text => join(":",@bit[1..7]));
}

### control interface-actions
sub cntl {
  my $i;
  my $o = 0;
  $set = $_[0];
  #print "ACTION: \$set=$set, \$bit[1]=$bit[1], \$bit[$set]=$bit[$set] \$bit[\$set-10]=" . ($bit[$set-10]) . "\n";
  if ($set < 10 && $bit[$set] == 1 && $bit[1] == 0 || $set >= 10 && $bit[$set - 10] == 1 && $bit[1] == 1) { # switch $set off
    #print "off: \$set=$set, \$bit[1]=$bit[1], \$bit[$set]=$bit[$set] \$bit[\$set-10]=" . ($bit[$set-10]) . "\n";
    if ($set >= 10) { $set = $set - 10; }
    $bit[$set] = 0;
    print CNTL "$set";
    if ($bit[1] == 1 && join("",@bit[2..6]) eq "00000") { $bit[1] = 0; print CNTL "1"; }
  } else { #switch $set on
    #print "on: \$set=$set, \$bit[1]=$bit[1], \$bit[$set]=$bit[$set] \$bit[\$set-10]=" . ($bit[$set-10]) . "\n";
    if ($set >= 10) {
      $set = $set - 10;
      if ($bit[1] != 1) {
        $bit[1] = 1;
        if (join("",@bit[2..6]) ne "00000") {
          for ($i = 2; $i <= 6; $i++) { if ($bit[$i] == 1) { $bit[$i] = 0; print CNTL "$i"; } }
        }
        print CNTL "1";
      }
    } else {
      if ($bit[1] == 1) {
        $bit[1] = 0;
        if (join("",@bit[2..6]) ne "00000") {
          for ($i = 2; $i <= 6; $i++) { if ($bit[$i] == 1) { $bit[$i] = 0; print CNTL "$i"; } }
        }
        print CNTL "1";
      }
    }
    $bit[$set] = 1;
    print CNTL "$set";
  }

  &update_relief;
  &update_statusbar;
}

### stop everything
sub emerg_stop {
  my $i;
  print CNTL "0";
  for ($i = 1; $i <= 6; $i ++) {
    $bit[$i] = 0;
  }
  if ($bit[7] == 1) { print CNTL "7"; }
  &update_relief;
  &update_statusbar;
}


### connecting to the interface
open(CNTL,"|./paico >/dev/null") || die $!;
select((select(CNTL), $| = 1)[0]); # w/o buffering

### set everything virtually off
@bit = (0,0,0,0,0,0,0,0);


### the main-window
$main = MainWindow->new(-title => "RobotArm Control");
$main->resizable(0,0);

$mainframe = $main->Frame()->pack;

@side_left = qw(-side left);
@bframes = qw(-fill x -anchor n);

$buttons = $mainframe->Frame()->pack(qw/-side left -anchor n/);

### buttons
$grapscolor = "lightgreen";
$graps = $buttons->Frame(-bg => $grapscolor)->pack(@bframes);
$graps->Label(-text => "Grapples:", -bg => $grapscolor)->pack(qw/-fill x -side top/);
$graps2 = $graps->Frame()->pack;
$button[12] = $graps2->Button(-text => "open", -bg => $grapscolor, -command => sub {&cntl(12)})->pack(@side_left);
$button[2] = $graps2->Button(-text => "close", -bg => $grapscolor, -command => sub {&cntl(2)})->pack(@side_left);

$grapjointcolor = "lightblue";
$grapjoint = $buttons->Frame(-bg => $grapjointcolor)->pack(@bframes);
$grapjoint->Label(-text => "Grapjoint:", -bg => $grapjointcolor)->pack(qw/-side top/);
$grapjoint2 = $grapjoint->Frame()->pack;
$button[13] = $grapjoint2->Button(-text => "<--", -bg => $grapjointcolor, -command => sub {&cntl(13)})->pack(@side_left);
$button[3] = $grapjoint2->Button(-text => "-->", -bg => $grapjointcolor, -command => sub {&cntl(3)})->pack(@side_left);

$lightcolor = "lightpink";
$light = $buttons->Frame(-bg => $lightcolor)->pack(@bframes);
$button[7] = $light->Button(-text => "Light", -bg => $lightcolor, -command => sub {&cntl(7)})->pack;

$joint1color = "palegreen3";
$joint2color = "mediumpurple";
$joints = $buttons->Frame()->pack(@bframes);
$joint1 = $joints->Frame(-bg => $joint1color)->pack(qw/-fill x -side left/);
$joint1->Label(-text => "Joint1", -bg => $joint1color)->pack(qw/-side top/);
$button[14] = $joint1->Button(-text => "up", -bg => $joint1color, -command => sub {&cntl(14)})->pack;
$button[4] = $joint1->Button(-text => "down", -bg => $joint1color, -command => sub {&cntl(4)})->pack;
$joint2 = $joints->Frame(-bg => $joint2color)->pack(qw/-fill x -side right/);
$joint2->Label(-text => "Joint2", -bg => $joint2color)->pack(qw/-side top/);
$button[15] = $joint2->Button(-text => "up", -bg => $joint2color, -command => sub {&cntl(15)})->pack;
$button[5] = $joint2->Button(-text => "down", -bg => $joint2color, -command => sub {&cntl(5)})->pack;

$swivelcolor = "burlywood3";
$swivel = $buttons->Frame(-bg => $swivelcolor)->pack(@bframes);
$swivel->Label(-text => "Swivel Joint", -bg => $swivelcolor)->pack(qw/-side top/);
$swivel2 = $swivel->Frame()->pack;
$button[16] = $swivel2->Button(-text => "<--", -bg => $swivelcolor, -command => sub {&cntl(16)})->pack(@side_left);
$button[6] = $swivel2->Button(-text => "-->", -bg => $swivelcolor, -command => sub {&cntl(6)})->pack(@side_left);

$button[0] = $buttons->Button(-text => "emerg. STOP", -bg => "red", -command => \&emerg_stop)->pack(qw/-anchor n -fill x/);
$infobutton = $buttons->Button(-text => "Info", -bg => "white", -command => \&info)->pack(qw/-side left -anchor n -fill x/);
$buttons->Button(-text => "Quit", -bg => "black", -fg => "white", -command => sub { $main->destroy; })->pack(qw/-side bottom -anchor s -fill x/);

### the photo
$photoframe = $mainframe->Frame()->pack(-side => "right");
$photoframe->Photo('robotarm', -file => "robotarm.gif");
$canvas = $photoframe->Canvas(-width => 175, -height => 285)->pack;
$photo = $canvas->createImage(0, 0, -anchor => "nw", -image => 'robotarm');

### arrows
@arrow = (-arrowshape => [8, 11, 5]);
$canvas->create(qw/line 0 20 50 39 -arrow last -width 5 -fill/ => $grapscolor, @arrow);
$canvas->create(qw/line 0 68 67 63 -arrow last -width 5 -fill/ => $grapjointcolor, @arrow);
$canvas->create(qw/line 0 108 136 117 -arrow last -width 5 -fill/ => $lightcolor, @arrow);
$canvas->create(qw/line 0 161 134 147 -arrow last -width 5 -fill/ => $joint1color, @arrow);
$canvas->create(qw/line 0 178 94 220 -arrow last -width 5 -fill/ => $joint2color, @arrow);
$canvas->create(qw/line 0 220 60 247 -arrow last -width 5 -fill/ => $swivelcolor, @arrow);

### StatusBar
$statusbar = $photoframe->Label(-relief => "sunken", -borderwidth => 1, -text => "", -justify => "left", -anchor => w)->pack(qw/-fill x -anchor w -side bottom/);
&update_statusbar;

### Info-Box
$info = $main->DialogBox(-title => "Info", -buttons => ["OK"]);
$info->Label(-text => "Author: $author\nVersion: $version\n
Keybindings:
Grapples	open/close => <Insert>/<Home>
Grapjoint	left/right => <Delete>/<End>
Light		on/off => <l>
Joint 1		up/down => <Page_Up>/<Page_Down>
Joint 2		up/down => <Up>/<Down>
Swivel Joint	left/right => <Left>/Right>
emergency STOP	on => <space>", -justify => "left", -anchor => w)->pack;

### keybindings
$main->bind('<Left>' => sub {&cntl(16)});
$main->bind('<Right>' => sub {&cntl(6)});
$main->bind('<Up>' => sub {&cntl(15)});
$main->bind('<Down>' => sub {&cntl(5)});
$main->bind('<Page_Up>' => sub {&cntl(14)});
$main->bind('<Page_Down>' => sub {&cntl(4)});
$main->bind('<l>' => sub {&cntl(7)});
$main->bind('<Insert>' => sub {&cntl(12)});
$main->bind('<Home>' => sub {&cntl(2)});
$main->bind('<Delete>' => sub {&cntl(13)});
$main->bind('<End>' => sub {&cntl(3)});
$main->bind('<space>' => \&emerg_stop);
$main->bind('<Escape>' => sub { $main->destroy; });

### Starting the TK-widgets
MainLoop;

### Close the CNTL-handler
close(CNTL);


